@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #Standard python imports
-import art, ast, json, h5py, numpy as np, os, subprocess
+import ast, json, h5py, numpy as np, os, subprocess
 try:                import configparser
 except ImportError: import ConfigParser as configparser
 
@@ -13,7 +13,7 @@ from pyRing.utils      import check_NR_dir, F_mrg_Nagar, import_datafile_path, p
 
 def store_git_info(output):
 
-    git_info = open(os.path.join(input_par['output'], 'git_info.txt'), 'w')
+    git_info = open(os.path.join(output, 'git_info.txt'), 'w')
     pipe1 = str(subprocess.Popen("git branch | grep \* | cut -d ' ' -f2", shell=True, stdout=subprocess.PIPE).stdout.read())[2:-1]
     pipe2 = str(subprocess.Popen("git log --pretty=format:'%H' -n 1 ",    shell=True, stdout=subprocess.PIPE).stdout.read())[2:-1]
     git_info.write('pyRing\nbranch: {}\t commit: {}\n'.format(pipe1, pipe2))
@@ -141,6 +141,7 @@ def read_config(Config, config_file):
                'injection-approximant'      : '',
                'inject-n-ds-modes'          : {"s": 0, "v": 0, "t": 1},
                'inject-area-quantization'   : 0,
+               'inject-braneworld'          : 0,
                'inject-charge'              : 0,
                'injection-scaling'          : 1.0,
                'injection-T'                : 64.0,
@@ -179,6 +180,7 @@ def read_config(Config, config_file):
                'domega-tgr-modes'           : None,
                'dtau-tgr-modes'             : None,
                'area-quantization'          : 0,
+               'braneworld'                 : 0,
                'tau-AQ'                     : 0,
                'prior-reweight'             : 0,
                'ParSpec'                    : 0,
@@ -302,6 +304,10 @@ def read_config(Config, config_file):
     if not((input_par['noise-averaging-method']=='mean') or (input_par['noise-averaging-method']=='median')):
         raise ValueError('Unknown noise-averaging method selected.')
 
+    if(input_par['spheroidal']):
+        try:    import pykerr
+        except: raise ValueError("The `spheroidal` option relies on the `pykerr` package; importing the latter however failed with error: {}.\nThe probable cause is that the `pykerr` package is not installed: you can solve this by pip-installing locally.\nThe reason why this is not done automatically by the package is that `pykerr` has not been deployed on conda, hence the conda-build fails and `pykerr` cannot be registered as a requirement in the `pyRing` package.".format(traceback.print_exc()))
+
     return input_par
 
 def read_injection_parameters(input_par, Config):
@@ -355,6 +361,7 @@ def read_injection_parameters(input_par, Config):
                                         'cosiota'         : 1.0,
                                         'phi'             : 0.0,
                                         'alpha'           : 8.*np.pi, # Bekenstein value for area quantization
+                                        'beta'            : -0.4, # Typical value for braneworld parameter
                                         'Q'               : 0.0, # Absolute value of the adimensional electric charge,
                                         'kerr-amplitudes' : {(2,2,-2,0): 0.0, (2,2,-1,0): 0.0, (2,2,0,0): 0.0, (2,2,1,0): 0.0, (2,2,2,0): 1.0},
                                         'kerr-phases'     : {(2,2,-2,0): 0.0, (2,2,-1,0): 0.0, (2,2,0,0): 0.0, (2,2,1,0): 0.0, (2,2,2,0): 0.0},
@@ -767,8 +774,8 @@ to be intended as part of the default value.
 
                noise-averaging-method     Option to perform the noise averaging using different methods. Available options: ['mean', 'median']. Deafult: 'mean'.
                fft-acf                    Flag to compute the ACF through an FFT. Default: 1.
-               acf-simple-norm            Option to FIXME Default: False.
-               no-lognorm                 Option to FIXME Default: False.
+               acf-simple-norm            Option to activate the simple 1/N norm of the ACF estimator. With this norm, the ACF estimator is biased, but reduces the influence of the terms with large lag, where the ACF variance is larger (see arXiv:2107.05609 for references). Instead by default, at lag k, we use the unbiased estimator with norm 1/(N-k). Default: False.
+               no-lognorm                 Flag to omit the likelihood normalisation. Default: 0.
                truncate                   Flag to computed the likelihood only on data starting at t0. Default: 1.
                analysis-duration          Option used to set seglen (in seconds) if truncating. Default: 0.1.
 
@@ -815,7 +822,7 @@ to be intended as part of the default value.
                # Damped sinusoids parameters. #
                #==============================#
 
-               n-ds-modes                 Number of scalar, vector and tensor damped sinuoids to be used for 'Damped-sinusoids' template. Default: '{"s": 0, "v": 0, "t": 1}'.
+               n-ds-modes                 Number of scalar, vector and tensor damped sinusoids to be used for 'Damped-sinusoids' template. Default: '{"s": 0, "v": 0, "t": 1}'.
                ds-ordering                Sort the modes according to increasing values of this parameter, relevant for damped-sinusoid waveforms. Available options: ['freq', 'tau', 'amp']. Default: 'freq'.
                ds-amp-flat-prior          Flag to select a flat prior on the damped-sinusoids amplitude. Default: 0.
 
@@ -823,7 +830,7 @@ to be intended as part of the default value.
                # Kerr GR parameters. #
                #=====================#
 
-               kerr-modes                 Modes to be used in 'Kerr' template. Default: [(2,2,2,0)] (syntax: [(s,l,m,n)]).
+               kerr-modes                 Modes to be used in 'Kerr' template. Default: [(2,2,2,0)] (syntax: [(s,l,m,n)]). Note: s \=2 modes correspond to non-tensorial QNMs coupling to the tensorial gravitational degrees of freedom.
                reference-amplitude        Option to set a reference amplitude (e.g. 1E-21) in place of the M_f/D_L waveform prefactor (in geometrised units) since this factor is completely degenerate with modes free amplitudes. Overwrites sampling in distance. Default: 0.0 (not active).
                spheroidal                 Flag to allow for spheroidal decomposition of the waveform. Default: 0.
                qnm-fit                    Flag to use fits for QNM complex frequencies, instead of interpolating numerical relativity data directly. Available only up to n=2 and l=4, for s=2. If 0, requires a PYRING_PREFIX to be set. Default: 1.
@@ -839,6 +846,7 @@ to be intended as part of the default value.
                domega-tgr-modes           Modes on which to sample on deviation from GR prediction in QNM frequency. Example of expected syntax: [(2,2,0)]. Default: None.
                dtau-tgr-modes             Modes on which to sample on deviation from GR prediction in QNM frequency. Example of expected syntax: [(2,2,0)]. Default: None.
                area-quantization          Flag to allow for area-quantization test. Default: 0.
+               braneworld                 Flag to allow for braneworld corrections to the Kerr spectrum. Default: 0.
                tau-AQ                     Flag to sample on tau in area-quantization test. Default: 0.
                prior-reweight             Flag used in area-quantization tests to reweight the prior for injections. Default: 0.
                ParSpec                    Flag to sample GR deviations using the parametrised spinning formalism. Default: 0.
@@ -877,13 +885,16 @@ to be intended as part of the default value.
                # For more info see: https://johnveitch.github.io/cpnest #
                #========================================================#
 
-               verbose                    Option to set the output verbosity of the sampler. Available options: FIXME MISSING. Default: 2.
-               poolsize                   Option to set the number of pools in the CPNest algorithm. Default: 128.
-               nthreads                   Option to set the number of parallel threads to be used by CPNest. Default: 1.
-               nlive                      Option to set the number of live points (main feature controlling the accuracy of the sampling) to be used by CPNest. Default: 1024.
-               maxmcmc                    Option to set the number of maximum Markov Chain Monte Carlo steps at each sample (main feature limiting the autocorrelation legth of the samples) to be used by CPNest. Default: 1024.
-               seed                       Option to set the random seed to be used by CPNest. Default: 1234.
-               resume                     Flag to allow CPNest to resume the run after it was killed. Default: 1.
+               verbose                      Option to set the output verbosity of the sampler. Available options: FIXME MISSING. Default: 2.
+               poolsize                     Option to set the number of pools in the CPNest algorithm. Default: 128.
+               nthreads                     Option to set the number of parallel threads to be used by CPNest. Default: 1.
+               nlive                        Option to set the number of live points (main feature controlling the accuracy of the sampling) to be used by CPNest. Default: 1024.
+               maxmcmc                      Option to set the number of maximum Markov Chain Monte Carlo steps at each sample (main feature limiting the autocorrelation legth of the samples) to be used by CPNest. Default: 1024.
+               seed                         Option to set the random seed to be used by CPNest. Default: 1234.
+               resume                       Flag to allow CPNest to resume the run after it was killed. Default: 1.
+               periodic_checkpoint_interval Option to checkpoint the sampler every `periodic_checkpoint_interval` seconds.  Default: 900.
+               
+               
 
        *******************************************************
        * Parameters to be passed to the [Injection] section. *
@@ -901,8 +912,11 @@ to be intended as part of the default value.
                 imr-samples               File containing the posterior samples from an IMR run to compare against ringdown values in plots. Needs to be either a full posterior in the GWTC-1 format or the marginalised posterior on Mf-af, with two columns and the ['Mf', 'af'] header.
 
 """
-
-my_art = art.text2art("            Launching     pyRing") # Return ASCII text (default font)
+                                                     
+try:
+    import art
+    my_art = art.text2art("            Launching     pyRing") # Return ASCII text (default font)
+except: print("* Warning: The `art` package could not be imported. Please consider installing it locally for best visual renditions. The cause of this not being taken care of automatically by the `pyRing` package is that the `art` package is not deployed on conda, hence the conda-build fails and `art` cannot be listed as a requirement for the `pyRing` package.")
 
 __ascii_art__ = """\n\n \u001b[\u001b[38;5;39m
                                          @.
