@@ -3,7 +3,7 @@
 from __future__     import division
 try:                import configparser
 except ImportError: import ConfigParser as configparser
-import h5py, matplotlib.pyplot as plt, numpy as np, os, sys
+import h5py, matplotlib.pyplot as plt, math, numpy as np, os, sys
 
 #LVC imports
 from lalinference                import DetFrameToEquatorial, EquatorialToDetFrame
@@ -21,8 +21,8 @@ def inject_ringdown_signal(lenstrain, tstart, length, ifo, triggertime, **kwargs
         Main function to set an injection using one of the analytical ringdown templates available.
         Handles parameters common to all templates.
     """
-
-    time_wf = np.linspace(0,length,lenstrain)
+    
+    time_wf = np.linspace(0,length,lenstrain+1)[:-1]
 
     if (kwargs['injection-approximant']=='Damped-sinusoids'):
         wf_model = damped_sinusoids_injection(**kwargs)
@@ -67,10 +67,10 @@ def inject_ringdown_signal(lenstrain, tstart, length, ifo, triggertime, **kwargs
     hs, hvx, hvy, hp, hc = hs*scaling, hvx*scaling, hvy*scaling, hp*scaling, hc*scaling
     wave       = project(hs, hvx, hvy, hp, hc, detector, ra, dec, psi, tM_gps)
     time_delay = lal.ArrivalTimeDiff(detector.location, ref_det.location, ra, dec, tM_gps)
-    trig_idx   = int((triggertime+time_delay-tstart)*srate)
 
-    for i in range(0, len(wave)-trig_idx):
-        h[trig_idx+i] = wave[i]
+    trig_idx   = math.ceil((triggertime+time_delay-tstart)*srate)
+
+    for i in range(0, len(wave)-trig_idx): h[trig_idx+i] = wave[i]
 
     return h
 
@@ -106,6 +106,7 @@ def kerr_injection(**kwargs):
     domegas          = kwargs['injection-parameters']['kerr-domegas']
     dtaus            = kwargs['injection-parameters']['kerr-dtaus']
     area_flag        = kwargs['inject-area-quantization']
+    braneworld_flag  = kwargs['inject-braneworld']
     charge_flag      = kwargs['inject-charge']
     spheroidal       = kwargs['spheroidal']
     Amp_non_prec_sym = kwargs['amp-non-prec-sym']
@@ -135,6 +136,10 @@ def kerr_injection(**kwargs):
     elif(charge_flag):
         TGR_parameters['Q'] = Q
         sys.stdout.write('* Injecting a KN waveform. Q: {}'.format(TGR_parameters['Q']))
+    elif(braneworld_flag):
+        TGR_parameters['beta'] = kwargs['injection-parameters']['beta']
+        sys.stdout.write('Injecting a braneworld waveform. beta: {}'.format(TGR_parameters['beta']))
+
     if not(af**2 + Q**2 < 1):
         raise ValueError("The selected values of charge and spin break the extremality limit (spin = {spin}, charge = {charge} : af^2 + Q^2 = {tot}).".format(spin=af, charge=Q, tot = af**2 + Q**2))
 
@@ -152,7 +157,8 @@ def kerr_injection(**kwargs):
                          Spheroidal       = spheroidal      ,
                          amp_non_prec_sym = Amp_non_prec_sym,
                          AreaQuantization = area_flag       ,
-                         charge           = charge_flag     )
+                         charge           = charge_flag     ,
+                         braneworld       = braneworld_flag )
 
     return wf_model
 
