@@ -4,7 +4,7 @@ import matplotlib
 matplotlib.use("pdf")
 import corner, h5py, matplotlib.pyplot as plt, numpy as np, os, sys, traceback, warnings
 from matplotlib.ticker import FormatStrFormatter
-from scipy.stats       import anderson, gaussian_kde
+from scipy.stats       import kstest, gaussian_kde
 from scipy.signal      import tukey
 
 try:
@@ -95,7 +95,7 @@ def FindHeightForLevel(inArr, adLevels):
 
     return np.sort(adHeights)
 
-def plot_contour(samples_stacked, level=[0.9], linest = 'dotted', label= None, color='k', line_w=1.2):
+def plot_contour(samples_stacked, level=[0.9], linest = 'dotted', label= None, color='k', line_w=1.2, plot_legend=1, zorder=None):
 
     warnings.filterwarnings('ignore', category=RuntimeWarning)
 
@@ -106,17 +106,19 @@ def plot_contour(samples_stacked, level=[0.9], linest = 'dotted', label= None, c
     grid_coords = np.append(X.reshape(-1,1),Y.reshape(-1,1),axis=1)
     pdf         = kde(grid_coords.T)
     pdf         = pdf.reshape(128,128)
+    pdf[np.where(pdf==0.)] = 1.e-100
+
     hs  = []
     lgs = []
     for l in level:
-        cntr       = plt.contour(X,Y,np.log(pdf),levels = np.sort(FindHeightForLevel(np.log(pdf),[l])), colors=color, linewidths=line_w, linestyles=linest)
-        h,_ = cntr.legend_elements()
-        hs.append(h[0])
-        if not(label==None):
-            lgs.append(r'${0} - {1} \% \, CI$'.format(label,int(l*100.)))
-        else:
-            lgs.append(r'${0} \% \, CI$'.format(int(l*100.)))
-    plt.legend([h_x for h_x in hs], [lg for lg in lgs], loc='upper left')
+        if zorder is not None: cntr = plt.contour(X,Y,np.log(pdf),levels = np.sort(FindHeightForLevel(np.log(pdf),[l])), colors=color, linewidths=line_w, linestyles=linest)
+        else:                  cntr = plt.contour(X,Y,np.log(pdf),levels = np.sort(FindHeightForLevel(np.log(pdf),[l])), colors=color, linewidths=line_w, linestyles=linest, zorder=zorder)
+        if(plot_legend):
+            h,_ = cntr.legend_elements()
+            hs.append(h[0])
+            if not(label==None):  lgs.append(r'${0} - {1} \% \, CI$'.format(label,int(l*100.)))
+            else:                 lgs.append(r'${0} \% \, CI$'.format(int(l*100.)))
+    if(plot_legend): plt.legend([h_x for h_x in hs], [lg for lg in lgs], loc='upper left')
 
     warnings.filterwarnings('default', category=RuntimeWarning)
 
@@ -219,6 +221,7 @@ def plot_strain(get_waveform, dets, fixed_params, tgps, params = None, whiten_fl
     for i,d in enumerate(dets.keys()):
         detector = dets[d]
         ax = fig.add_subplot(nsub_panels,1,i+1)
+        plt.grid(False)
         if whiten_flag:
             
             if(whiten_method=='FD'):
@@ -243,7 +246,9 @@ def plot_strain(get_waveform, dets, fixed_params, tgps, params = None, whiten_fl
 
             wtleg      = 'whitened'
             label_y    = r'$\mathrm{s_{%s}(t)}$'%(d)
-            ax.set_ylim([-5,5])
+            if (np.max(np.abs(timeseries)) < 5.): ax.set_ylim([-5,5])
+            else:                                 ax.set_ylim([-10,10])
+        
         else:
             timeseries = detector.time_series
             time_axis  = detector.time-tevent
@@ -261,17 +266,18 @@ def plot_strain(get_waveform, dets, fixed_params, tgps, params = None, whiten_fl
             dt_regions       = np.percentile(np.array(dt_dict[d]),[5,50,95], axis=0)
             ax.plot(time_axis, waveform_regions[1], label='model '+wtleg, color='gold', lw=0.8)
             ax.fill_between(time_axis, waveform_regions[0], waveform_regions[2], facecolor='gold', lw=0.5, alpha=0.4)
+            
             try:
                 if(  whiten_method=='FD'):
-                    ax.set_xlim([dt-150*mf_time_prior*lal.MTSUN_SI, dt+80*mf_time_prior*lal.MTSUN_SI])
+                    ax.set_xlim([dt-150*mf_time_prior*lal.MTSUN_SI, dt+150*mf_time_prior*lal.MTSUN_SI])
                     ax.axvline(0.0, c='deeppink', linestyle='dashed', lw=1.0, label=r'$\mathrm{t_{start}}$')
                 elif(whiten_method=='TD'):
-                    ax.set_xlim([0.0, dt+150*mf_time_prior*lal.MTSUN_SI])
+                    if not whiten_flag: ax.set_xlim([dt-150*mf_time_prior*lal.MTSUN_SI, dt+150*mf_time_prior*lal.MTSUN_SI])
+                    else              : ax.set_xlim([dt-  0*mf_time_prior*lal.MTSUN_SI, dt+150*mf_time_prior*lal.MTSUN_SI])
             except (KeyError,configparser.NoOptionError, configparser.NoSectionError):
                 print("\nWarning: Failed to complete strain plot due to error: {}.".format(traceback.print_exc()))
         ax.legend(loc='upper left', prop={'size': 6})
         ax.set_xlabel('Time [s]')
-    plt.grid(False)
     plt.subplots_adjust(wspace=0, hspace=0)
     if not(wtleg==''): filename = filename + '_' + wtleg + '_{}'.format(whiten_method)
     plt.savefig(os.path.join(kwargs['output'],'Plots/Strains', filename+'.pdf'), bbox_inches='tight')
@@ -377,38 +383,40 @@ def plot_strain(get_waveform, dets, fixed_params, tgps, params = None, whiten_fl
                 plt.xlabel('Whitened residuals')
                 plt.savefig(os.path.join(kwargs['output'],'Plots/Strains','Histrogram_whitened_residuals_{}_{}.pdf'.format(d, whiten_method)), bbox_inches='tight')
 
-                # Now let's compute a quantitative measure of gaussianity.
-                AD_statistics = []
+                # Now let's compute a quantitative measure of gaussianity with zero mean and unit variance.
+                KS_p_values = []
+                p_value_threshold = 0.01
                 for i in range(len(model_waveforms[d])):
-                    AD_statistic, critical_values, significance_level = anderson(timeseries_whitened_TD[d][i]-model_waveforms[d][i], dist='norm')
-                    AD_statistics.append(AD_statistic)
+                    res_x = timeseries_whitened_TD[d][i]-model_waveforms[d][i]
+                    KS_statistic, p_value = kstest(res_x, "norm")
+                    KS_p_values.append(p_value)
 
-                AD_statistics = np.array(AD_statistics)
-                mask_outliers = AD_statistics > critical_values[-1]
-                N_outliers    = len(AD_statistics[mask_outliers])
-                len_tot       = len(AD_statistics)
-                n, bins, _    = plt.hist(AD_statistics)
+                KS_p_values   = np.array(KS_p_values)
+                mask_outliers = KS_p_values < p_value_threshold
+                N_outliers    = len(KS_p_values[mask_outliers])
+                len_tot       = len(KS_p_values)
+                n, bins, _    = plt.hist(KS_p_values)
                 bin_width     = bins[1] - bins[0]
                 integral      = bin_width * sum(n)
 
                 plt.figure()
-                plt.hist(AD_statistics, histtype='step', bins=nbins, stacked=True, fill=False, color='black', linewidth=2.0)
-                plt.axvline(critical_values[-1], label = 'Significance level: {}\nN outliers: {}/{}'.format(significance_level[-1], N_outliers, len_tot), color='darkred', linestyle='dashed', linewidth=1.5)
-                plt.xlabel('Anderson-Darling statistic')
+                plt.hist(KS_p_values, histtype='step', bins=nbins, stacked=True, fill=False, color='black', linewidth=2.0)
+                plt.axvline(p_value_threshold, label = 'Significance level: {}\nN outliers: {}/{}'.format(p_value_threshold, N_outliers, len_tot), color='darkred', linestyle='dashed', linewidth=1.5)
+                plt.xlabel('Kolmogorov–Smirnov p-values')
                 plt.title('Integral: {:.6f}'.format(integral))
                 plt.legend(loc='upper right')
-                plt.savefig(os.path.join(kwargs['output'],'Plots/Strains',' Histogram_Anderson_Darling_statistic_whitened_residuals_{}_{}.pdf'.format(d, whiten_method)), bbox_inches='tight')
+                plt.savefig(os.path.join(kwargs['output'],'Plots/Strains',' Histogram_Kolmogorov_Smirnov_test_whitened_residuals_{}_{}.pdf'.format(d, whiten_method)), bbox_inches='tight')
 
                 plt.figure()
-                plt.scatter(np.arange(0,len(AD_statistics)), AD_statistics, color='black', marker='x')
-                plt.axhline(critical_values[-1], label = 'Significance level: {}\nN outliers: {}/{}'.format(significance_level[-1], N_outliers, len_tot), color='darkred', linestyle='dashed', linewidth=1.5)
+                plt.scatter(np.arange(0,len(KS_p_values)), KS_p_values, color='black', marker='x')
+                plt.axhline(p_value_threshold, label = 'Significance level: {}\nN outliers: {}/{}'.format(p_value_threshold, N_outliers, len_tot), color='darkred', linestyle='dashed', linewidth=1.5)
                 plt.xlabel('Samples')
-                plt.ylabel('Anderson-Darling statistic')
+                plt.ylabel('Kolmogorov–Smirnov p-values')
                 plt.ylim([0,2])
                 plt.legend(loc='upper right')
-                plt.savefig(os.path.join(kwargs['output'],'Plots/Strains',' Scatter_Anderson_Darling_statistic_whitened_residuals_{}_{}.pdf'.format(d, whiten_method)), bbox_inches='tight')
+                plt.savefig(os.path.join(kwargs['output'],'Plots/Strains',' Scatter_Kolmogorov_Smirnov_test_whitened_residuals_{}_{}.pdf'.format(d, whiten_method)), bbox_inches='tight')
 
-                print('* Anderson-Darling statistic of whitened {} residuals gave {}/{} normality outliers (at {} % significance).\n'.format(d, N_outliers, len_tot, significance_level[-1]))
+                print('* A Kolmogorov–Smirnov test of whitened {} residuals gave {}/{} normality outliers (at {} % significance).\n'.format(d, N_outliers, len_tot, p_value_threshold*100))
 
     return
 
@@ -425,7 +433,7 @@ def SNR_plots(get_waveform, dets, fixed_params, tgps, params = None, **kwargs):
     tevent            = kwargs['trigtime']
     srate             = kwargs['sampling-rate']
     sky_frame         = kwargs['sky-frame']
-    seglen            = np.int(srate*kwargs['signal-chunksize'])
+    seglen            = int(srate*kwargs['signal-chunksize'])
     dt                = 1./srate
     freqs             = np.fft.rfftfreq(seglen, d=dt)
     df                = freqs[1] - freqs[0]
@@ -544,6 +552,27 @@ def SNR_plots(get_waveform, dets, fixed_params, tgps, params = None, **kwargs):
 
     return
 
+def global_corner(x, names, output):
+
+    samples = []
+    for xy in names: samples.append(np.array(x[xy]))
+    samples = np.transpose(samples)
+    mask    = [i for i in range(samples.shape[-1]) if not all(samples[:,i]==samples[0,i]) ]
+
+    fig = plt.figure(figsize=(10,10))
+    C   = corner.corner(samples[:,mask],
+                        quantiles     = [0.5, 0.95],
+                        labels        = names,
+                        color         = 'darkred',
+                        show_titles   = True,
+                        title_kwargs  = {"fontsize": 12},
+                        use_math_text = True
+                        )
+    plt.savefig(os.path.join(output,'Plots','Parameters', 'corner.png'), bbox_inches='tight')
+
+    return
+
+
 def mode_corner(samples,filename=None,**kwargs):
     
     fig = plt.figure(figsize=(10,10))
@@ -552,50 +581,75 @@ def mode_corner(samples,filename=None,**kwargs):
     
     return fig
 
-def f0_t0_gr(samples, Mf_s, af_s, Num_mode, **kwargs):
+def f0_t0_gr(samples_x, Mf_s, af_s, Num_mode, probs_flag=0, scatter_samples_flag=0, delta_QNM_plots=0, predictions_contour_flag=0, **kwargs):
+    
     init_plotting()
     output_dir = os.path.join(kwargs['output'], 'Plots')
     
     # Initialise structures.
     theoretical_values, labels    = [], []
     qnm_interpolants              = {}
-    i, Num_modes, delta_QNM_plots = 0, 0, 0
-    
-    #IMPROVEME: Colours for generic N modes, now it's hardcoded for 12 modes.
-    n_values = [0]
-    l_values = [2,3]
-    for l in l_values:
-        Num_modes += len(n_values)*(2*l+1)
+    i, Num_modes                  = 0, 0
+    n_values                      = [0,1]
+    l_values                      = [2,3]
+    for l in l_values: Num_modes += (2*l+1)
     print('\n* Total number of modes used to produce frequency-tau plots: {}\n'.format(Num_modes))
-    colors_manual= [ 'green', 'forestgreen', 'mediumaquamarine', 'springgreen', 'aquamarine', 'khaki', 'yellow','gold', 'orange', 'orangered', 'red', 'firebrick']
+
+    fig = plt.figure()
+    ax  = plt.axes()
+
+    # Plot the posteriors from the tensorial agnostic reconstruction.
+    pol = 't'
+    for i in range(kwargs['n-ds-modes'][pol]):
+        
+        samples = np.column_stack((samples_x['f_{}_{}'.format(pol,i)],1e3*samples_x['tau_{}_{}'.format(pol,i)]))
+        plot_contour(samples, level=[0.5, 0.9], linest = 'solid', color='k', line_w=0.8, plot_legend=0, zorder=1)
+        if(scatter_samples_flag): plt.scatter(samples[:,0], samples[:,1], cmap='seismic',  marker='.', alpha=0.3, s=2)
 
     # Plot the GR prediction for frequency and damping time coming from an IMR run.
-    fig=plt.figure()
-    ax = plt.axes()
     for n in n_values:
+        
+        #IMPROVEME: Palettes for generic N modes, now it's set for 2 sets of overtones modes.
+        if(n==0): palette = iter(matplotlib.cm.Spectral(np.linspace( 0, 1, Num_modes)))
+        if(n==1): palette = iter(matplotlib.cm.rainbow_r(np.linspace(0, 1, Num_modes)))
+        
         for l in l_values:
             for m in range(-l,l+1,1):
-                col = colors_manual[i]
+                
+                col = next(palette)
+                
                 if(kwargs['qnm-fit'] == 1):
                     qnm                               = QNM_fit(l,m,n)
                 else:
                     interpolate_freq, interpolate_tau = qnm_interpolate(2,l,m,n)
                     qnm_interpolants[(2,l,m,n)]       = {'freq': interpolate_freq, 'tau': interpolate_tau}
                     qnm                               = QNM(2,l,m,n,qnm_interpolants)
+                
                 f0 = [qnm.f(M,a)       for M,a in zip(Mf_s,af_s)]
                 t0 = [1e3*qnm.tau(M,a) for M,a in zip(Mf_s,af_s)]
                 theoretical_values.append([f0,t0])
                 labels.append(r'$(%d,%d,%d)$ '%(l,m,n))
-                f0_t0_IMR   = np.column_stack((f0,t0))
-                kde         = gaussian_kde(f0_t0_IMR.T)
-                x_flat      = np.r_[f0_t0_IMR[:,0].min():f0_t0_IMR[:,0].max():128j]
-                y_flat      = np.r_[f0_t0_IMR[:,1].min():f0_t0_IMR[:,1].max():128j]
-                X,Y         = np.meshgrid(x_flat,y_flat)
-                grid_coords = np.append(X.reshape(-1,1),Y.reshape(-1,1),axis=1)
-                pdf         = kde(grid_coords.T)
-                pdf         = pdf.reshape(128,128)
-                pdf[np.where(pdf==0.)] = 1.e-100
-                plt.contour(X,Y,np.log(pdf),levels = np.sort(FindHeightForLevel(np.log(pdf),[0.9])), colors=col, linewidths=2.0, linestyles='solid')
+                
+                if(predictions_contour_flag): plot_contour(samples, level=[0.9], linest = 'solid', color=col, line_w=1.2, plot_legend=0)
+
+                # Fine tune text positioning.
+                shift_f   = 10
+                shift_tau = 0.3
+                if(l==2 and m==-2         ): shift_tau += -0.1
+                if(l==3 and m==-2         ): shift_tau +=  0.1
+                if(l==3 and m==-3         ): shift_tau +=  0.2
+                if(l==2 and m==-2 and n==1): shift_tau += -0.1
+                if(l==2 and m==-2 and n==1): shift_f   +=  5.0
+                if(l==2 and m==-1 and n==1): shift_f   +=  3.0
+                if(l==2 and m==-1 and n==1): shift_tau += -0.1
+                if(m<0 and n==0           ): shift_f   +=  4.0
+
+                # Invert label positioning between l=2 and l=3.
+                if(l==2): shift_tau = -shift_tau
+                font_text = {'color':  'black', 'weight': 'normal', 'size': 5}
+                plt.plot(np.median(f0),         np.median(t0), color=col, marker='*', markersize=7,                    zorder=2)
+                plt.text(np.median(f0)-shift_f, np.median(t0)-shift_tau, r'$(%d,%d,%d)$ '%(l,m,n), fontdict=font_text, zorder=3)
+
                 i = i+1
                 if(delta_QNM_plots):
                     difference_freq = np.array([(samples[j,0]-f0[k])/f0[k] for k in range(len(f0)) for j in range(0,samples.shape[0])])
@@ -611,36 +665,28 @@ def f0_t0_gr(samples, Mf_s, af_s, Num_mode, **kwargs):
                     plt.ylabel(r'$\mathrm{P(\delta \tau_{%d%d%d}|D)}$'%(l,m,n))
                     plt.savefig(os.path.join(output_dir,'Parameters/dtau_{}{}{}_{}.png'.format(l,m,n,Num_mode)) ,bbox_inches='tight')
 
-    # Plot the freq-tau 2-D posterior on top of GR predictions.
-    kde         = gaussian_kde(samples.T)
-    x_flat      = np.r_[samples[:,0].min():samples[:,0].max():128j]
-    y_flat      = np.r_[samples[:,1].min():samples[:,1].max():128j]
-    X,Y         = np.meshgrid(x_flat,y_flat)
-    grid_coords = np.append(X.reshape(-1,1),Y.reshape(-1,1),axis=1)
-    pdf         = kde(grid_coords.T)
-    pdf         = pdf.reshape(128,128)
-    plt.scatter(samples[:,0], samples[:,1], cmap='seismic',  marker='.', alpha=0.3, s=2)
-    plt.contour(X, Y, np.log(pdf), levels = np.sort(FindHeightForLevel(np.log(pdf),[0.5,0.90])), colors='k', linestyles='solid')
 
     # Compute the probability that each of the modes is the one corresponding to the samples.
     # To understand why the kde evaluation on the samples, combined with the mean of the probability, is the correct procedure to compute this number, see TeX/Likelihood.tex
-    theoretical_values = np.array(theoretical_values)
-    probs              = np.array([kde(t) for t in theoretical_values])
+    if(probs_flag):
+        theoretical_values = np.array(theoretical_values)
+        probs              = np.array([kde(t) for t in theoretical_values])
 
-    mprob = np.array([np.mean(probs[i,:]) for i in range(len(colors_manual))])
-    mprob /= mprob.sum()
-    import matplotlib.patches as mpatches
-    patches = [mpatches.Patch(color=c, label = labels[i]+r'$p={} \%$'.format(int(100*mprob[i]))) for i,c in enumerate(colors_manual)]
+        mprob = np.array([np.mean(probs[i,:]) for i in range(len(colors_manual))])
+        mprob /= mprob.sum()
+        import matplotlib.patches as mpatches
+        patches = [mpatches.Patch(color=c, label = '$(\ell,m,n) = $'+labels[i]) for i,c in enumerate(colors_manual)]
 
-    l = ax.legend(handles = patches,
-                  fancybox       = True,
-                  loc            = "upper right",
-                  borderaxespad  = 0.,
-                  fontsize       = 7)
+        l = ax.legend(handles = patches,
+                      fancybox       = True,
+                      loc            = "upper left",
+                      borderaxespad  = 0.,
+                      fontsize       = 6)
 
     ax.set_xlabel(r'$\mathrm{f\,(Hz)}$')
     ax.set_ylabel(r'$\mathrm{\tau\, (ms)}$')
-    plt.savefig(os.path.join(output_dir,'Parameters/frequency_tau_{}.pdf'.format(Num_mode)) ,bbox_inches='tight')
+    plt.savefig(os.path.join(output_dir,'Parameters/frequency_tau_{}.pdf'.format(Num_mode)), bbox_inches='tight')
+
     return fig
 
 def Kerr_intrinsic_corner(x, **input_par):
@@ -663,25 +709,41 @@ def Kerr_intrinsic_corner(x, **input_par):
 
 def Kerr_intrinsic_alpha_corner(x, **input_par):
 
-    # Corner plot of final mass, final spin, and alpha
-    try:
-        pos             = np.column_stack((x['Mf'], x['af'], x['alpha']))
-        injected_values = None
-        if (input_par['inject-area-quantization']):
-            injected_values = [input_par['injection-parameters']['Mf'], input_par['injection-parameters']['af'], input_par['injection-parameters']['alpha']]
+    # Corner plot of final mass, final spin, and alpha.
+    pos             = np.column_stack((x['Mf'], x['af'], x['alpha']))
+    injected_values = None
+    if (input_par['inject-area-quantization']):
+        injected_values = [input_par['injection-parameters']['Mf'], input_par['injection-parameters']['af'], input_par['injection-parameters']['alpha']]
 
-        mode_corner(  pos,
-                    labels        = [r'$M_f (M_{\odot})$',
-                                     r'$a_f$'            ,
-                                     r'$\alpha$'      ],
-                    quantiles     = [0.05, 0.5, 0.95],
-                    show_titles   = True,
-                    title_kwargs  = {"fontsize": 12},
-                    use_math_text = True,
-                    truths        = injected_values,
-                    filename      = os.path.join(input_par['output'],'Plots/Parameters/Kerr_intrinsic_alpha_corner.png'))
-    except(ValueError):
-        pass
+    mode_corner(  pos,
+                labels        = [r'$M_f (M_{\odot})$',
+                                 r'$a_f$'            ,
+                                 r'$\alpha$'      ],
+                quantiles     = [0.05, 0.5, 0.95],
+                show_titles   = True,
+                title_kwargs  = {"fontsize": 12},
+                use_math_text = True,
+                truths        = injected_values,
+                filename      = os.path.join(input_par['output'],'Plots/Parameters/Kerr_intrinsic_alpha_corner.png'))
+
+def Kerr_intrinsic_braneworld_corner(x, **input_par):
+    
+    # Corner plot of final mass, final spin, and beta.
+    pos             = np.column_stack((x['Mf'], x['af'], x['beta']))
+    injected_values = None
+    if (input_par['inject-braneworld']):
+        injected_values = [input_par['injection-parameters']['Mf'], input_par['injection-parameters']['af'], input_par['injection-parameters']['beta']]
+    
+    mode_corner(  pos,
+                labels        = [r'$M_f (M_{\odot})$',
+                                 r'$a_f$'            ,
+                                 r'$\beta$'      ],
+                quantiles     = [0.05, 0.5, 0.95],
+                show_titles   = True,
+                title_kwargs  = {"fontsize": 12},
+                use_math_text = True,
+                truths        = injected_values,
+                filename      = os.path.join(input_par['output'],'Plots/Parameters/Kerr_intrinsic_braneworld_corner.png'))
 
 def MMRDNS_intrinsic_corner(x, **input_par):
 
