@@ -9,9 +9,26 @@ import numpy as np, pkg_resources, os, scipy.linalg as sl, traceback, warnings
 import lal
 
 #Package internal imports
-try:    import surfinBH
-except: warnings.warn("* The `surfinBH` package is not automatically installed due to possible conflicts. If you wish to use its functionalities, it needs to be installed separately.")
+try:
+    import surfinBH
 
+    def final_state_surfinBH(Mtot, q, chi1, chi2, f_ref):
+
+        fit = surfinBH.LoadFits('NRSur7dq4Remnant')
+        #Adapt q to surrogate conventions.
+        if(q < 1.): q = 1./q
+        #This is the orbital frequency (hence the missing factor of 2, since f_ref is the GW frequency) in units of rad/M (seehttps://github.com/vijayvarma392/surfinBH/blob/master/examples/example_7dq4.ipynb for more info).
+        omega_ref = np.pi*f_ref/Mtot
+
+        Mf_sBH, _ = fit.mf(q, chi1, chi2, omega0=omega_ref)
+        af_sBH, _ = fit.chif(q, chi1, chi2, omega0=omega_ref)
+        Mf        = Mtot*Mf_sBH
+        af        = np.sqrt(af_sBH[0]**2+af_sBH[1]**2+af_sBH[2]**2)
+
+        return Mf, af
+
+except:
+    warnings.warn("* The `surfinBH` package is not automatically installed due to possible conflicts. If you wish to use its functionalities, it needs to be installed separately.")
 
 def print_section(name):
 
@@ -69,9 +86,6 @@ def check_NR_dir():
         raise Exception("pyRing supports NR injections using the LVC-NR injection infrastructure. If you wish to inject NR simulations, please clone the LVC-NR injection infrastructure repository, located here: https://git.ligo.org/waveforms/lvcnr-lfs \nFor tutorials and info on how to use the LVC NR injection infrastructure see:\n - https://git.ligo.org/sebastian-khan/waveform-f2f-berlin/blob/master/notebooks/2017WaveformsF2FTutorial_NRDemo.ipynb \n - https://www.lsc-group.phys.uwm.edu/ligovirgo/cbcnote/Waveforms/NR/InjectionInfrastructure \n - https://arxiv.org/pdf/1703.01076.pdf")
     return
 
-def m1_from_m_q(m, q): return m*q/(1+q)
-def m2_from_m_q(m, q): return m/(1+q)
-
 def review_warning():
     print("* Warning: You are using a code block which is not reviewed. Non-reviewed code cannot be used for producing LVC results.")
 
@@ -103,7 +117,26 @@ def qnm_interpolate_KN(s,l,m,n):
     except:
         raise Exception("Loading KN data failed. Exiting.".format(traceback.print_exc()))
 
-    coords = np.column_stack((af,Q))
+    coords   = np.column_stack((af,Q))
+    interp_r = LinearNDInterpolator(coords, w_r)
+    interp_i = LinearNDInterpolator(coords, w_i)
+
+    return interp_r, interp_i
+
+def qnm_interpolate_braneworld(s,l,m,n):
+    print('\nPerforming interpolation of ringdown Braneworld complex frequencies of {}{}{}{} mode.'.format(s,l,m,n))
+    assert not(np.abs(m) > l), "QNM interpolation: m cannot be greater than l in modulus."
+    assert (s==0 or s==1 or s==2), "QNM interpolation: supported s values are [0,1,2] ({} was passed)."
+    try:
+        PYRING_PREFIX = set_prefix()
+        if (m<0):
+            af, qf, w_r, w_i = np.loadtxt(os.path.join(PYRING_PREFIX,'data/NR_data/Braneworld/s{}l{}'.format(s, l),'n{}l{}mm{}.dat'.format(n, l, np.abs(m))), unpack=True)
+        else:
+            af, qf, w_r, w_i = np.loadtxt(os.path.join(PYRING_PREFIX,'data/NR_data/Braneworld/s{}l{}'.format(s, l),'n{}l{}m{}.dat'.format(n, l, m)), unpack=True)
+    except:
+        raise Exception("Loading Braneworld data failed. Exiting.".format(traceback.print_exc()))
+
+    coords   = np.column_stack((af, qf))
     interp_r = LinearNDInterpolator(coords, w_r)
     interp_i = LinearNDInterpolator(coords, w_i)
 
@@ -196,21 +229,6 @@ def compute_SNR_FD(data, template, psd, df):
 
     return inner_product_FD(data, template, psd, df)/np.sqrt(inner_product_FD(template, template, psd, df))
 
-def final_state_surfinBH(Mtot, q, chi1, chi2, f_ref):
-
-    fit = surfinBH.LoadFits('NRSur7dq4Remnant')
-    #Adapt q to surrogate conventions.
-    if(q < 1.): q = 1./q
-    #This is the orbital frequency (hence the missing factor of 2, since f_ref is the GW frequency) in units of rad/M (seehttps://github.com/vijayvarma392/surfinBH/blob/master/examples/example_7dq4.ipynb for more info).
-    omega_ref = np.pi*f_ref/Mtot
-
-    Mf_sBH, _ = fit.mf(q, chi1, chi2, omega0=omega_ref)
-    af_sBH, _ = fit.chif(q, chi1, chi2, omega0=omega_ref)
-    Mf        = Mtot*Mf_sBH
-    af        = np.sqrt(af_sBH[0]**2+af_sBH[1]**2+af_sBH[2]**2)
-
-    return Mf, af
-
 def railing_check(samples, prior_bins, tolerance):
     hist, bin_edges = np.histogram(samples, bins=prior_bins, density=True)
 
@@ -250,10 +268,12 @@ def EsGB_corrections(name):
         raise ValueError("Currently EdsGB corrections only support the (l,m,n)=(2,2,0) mode deviation.")
     return corr
 
-def mtot(m1,m2):         return m1+m2
-def mc(m1,m2):           return (m1*m2)**(3./5.)/(m1+m2)**(1./5.)
-def q(m1,m2):            return [(np.minimum(m1,m2))/(np.maximum(m1,m2)), (np.maximum(m1,m2))/(np.minimum(m1,m2))]  #Return both of the q conventions
-def eta(m1,m2):          return (m1*m2)/(m1+m2)**2   #eta=q/(1+q)**2
+def mtot(m1,m2)        : return m1+m2
+def mc(m1,m2)          : return (m1*m2)**(3./5.)/(m1+m2)**(1./5.)
+def q(m1,m2)           : return [(np.minimum(m1,m2))/(np.maximum(m1,m2)), (np.maximum(m1,m2))/(np.minimum(m1,m2))]  #Return both of the q conventions
+def eta(m1,m2)         : return (m1*m2)/(m1+m2)**2   #eta=q/(1+q)**2
+def m1_from_m_q(m, q)  : return m*q/(1+q)
+def m2_from_m_q(m, q)  : return m/(1+q)
 def m1_from_mc_q(mc, q): return mc*((1+q)**(1./5.))*q**(-3./5.)
 def m2_from_mc_q(mc, q): return mc*((1+q)**(1./5.))*q**(2./5.)
 def chi_eff(q, a1, a2):                       #eta=0.25 -> chi_1=chi_2=0.5 -> chi_eff=aritmetic mean of (a1,a2)
